/* game.cc
   Contains all information about the current game

   Copyright (C) 2000  Mathias Broxvall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "general.h"
#include "game.h"

using namespace std;

Game::Game() {
  nColors=8;
  isComputer=0;
}

void Game::reset() {
  int i,j;

  finished=0;
  nAnswers=0;
	for(j=0;j<4;j++)
	  code[j] = 0;	
  for(i=0;i<32;i++)
	for(j=0;j<4;j++)
	  answers[i][j] = 0;
  for(i=0;i<32;i++)
	blacks[i]=whites[i]=0;

  bestValue=-1;
  thoughts=0;
  memset(possible,1,sizeof(char)*12*12*12*12);
  bestMove[0]=bestMove[1]=bestMove[2]=bestMove[3]=0;
}

void Game::computeBW(int code[4],int answer[4],int &black,int &white) {
  int codeUsed[4]={0,0,0,0},answerUsed[4]={0,0,0,0};
  int i,j;
  black=white=0;
  for(i=0;i<4;i++)
	if(answer[i] == code[i]) {black++; codeUsed[i]=1; answerUsed[i]=1; }
  for(i=0;i<4;i++)
	if(!answerUsed[i])
	  for(j=0;j<4;j++)
		if(!codeUsed[j] && answer[i] == code[j]) {white++; codeUsed[j]=1; answerUsed[i]=1; break; }
}

void Game::think() {
  int i;
  int move[4],m,m2;
  int code[4];
  int value;

  /* Find a possible move */
  m = (rand() % (nColors*nColors*nColors*nColors)); int2move(m,move);
  do {
	m=(m+1) % (nColors*nColors*nColors*nColors); int2move(m,move);
	if(possible[m]) {
	  if(type == TYPE_AUTOMATA && (rand()%200) < 10-nAnswers) break; // he forgets to check if it realy is possible
	  else possible[m] = test(move);
	}
  } while(!possible[m]);

  for(i=0;i<4;i++) answers[nAnswers][i]=move[i]; 

  if(type == TYPE_AI) {
	if(!(move[0]==bestMove[0] && move[1]==bestMove[1] && 
		 move[2]==bestMove[2] && move[3]==bestMove[3])) {
	  value=0;
	  nAnswers++;
	  for(blacks[nAnswers-1]=0;blacks[nAnswers-1]<4;blacks[nAnswers-1]++)
		for(whites[nAnswers-1]=0;whites[nAnswers-1]<4;whites[nAnswers-1]++)
		  if(blacks[nAnswers-1]+whites[nAnswers-1] <= 4) {
			for(m2=0;m2<nColors*nColors*nColors*nColors;m2++)
			  if(possible[m2]) {
				int2move(m2,move);
				if(test(move)) value++;
			  }	
		  }
	  nAnswers--;
	  blacks[nAnswers]=whites[nAnswers]=0;
	  if(bestValue == -1 || value < bestValue) {
		printf("%d ",value); fflush(stdout);
		bestValue=value;
		for(i=0;i<4;i++) bestMove[i] = answers[nAnswers][i];
	  } else
		for(i=0;i<4;i++) answers[nAnswers][i] = bestMove[i];
	}
  } else 
	for(i=0;i<4;i++) bestMove[i] = move[i];
  thoughts++;

}
void Game::commit() {
  int i;

  if(type == TYPE_AI)
	printf("best %d\n",bestValue);
  for(i=0;i<4;i++) answers[nAnswers][i]=bestMove[i]; 
  computeBW(code,answers[nAnswers],blacks[nAnswers],whites[nAnswers]);
  if(blacks[nAnswers] == 4) finished=1;
  else {
	nAnswers++;
	thoughts=0;
	bestValue=-1;
  }
}
int Game::test(int sol[4]) {
  int i,b,w;
  
  for(i=0;i<nAnswers;i++) {
	computeBW(sol,answers[i],b,w);
	if(b != blacks[i] || w != whites[i]) return 0;
  }
  return 1;
}
int Game::move2int(int move[4]) { return move[0]*(nColors*nColors*nColors)+move[1]*(nColors*nColors)+move[2]*nColors+move[3]; }
void Game::int2move(int m,int move[4]) {
  move[3]=m % nColors; m /= nColors;
  move[2]=m % nColors; m /= nColors;
  move[1]=m % nColors; m /= nColors;
  move[0]=m % nColors; m /= nColors;
}
