/* setupMode.h
   Sets up the game parameters

   Copyright (C) 2000  Mathias Broxvall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "general.h"
#include "gameMode.h"
#include "setupMode.h"
#include "game.h"
#include "mainMode.h"

using namespace std;

SDL_Surface *SetupMode::background;
SDL_Surface *SetupMode::title;
SDL_Surface *SetupMode::ply1,*SetupMode::ply2,*SetupMode::code,*SetupMode::nColors;
SDL_Surface *SetupMode::ok, *SetupMode::ok2, *SetupMode::type,*SetupMode::startGame,*SetupMode::startGame2;
SetupMode *SetupMode::setupMode;

#define P1_TYPE -1
#define P1_CODE 0
#define P1_COLORS 1

#define P2_TYPE 3
#define P2_CODE 4
#define P2_COLORS 5
#define START 7
#define QUIT 8
#define CODE_OK 7

char *plyTypeNames[5] = {"Human","Cyborg","Automata","Computer","AI"};

void SetupMode::init() {
  background = loadImage("setupBG.png");
  setupMode = new SetupMode();
  title = loadImage("mindless.png");
  ply1 = loadImage("player1.png");
  ply2 = loadImage("player2.png");
  code = loadImage("code.png");
  nColors = loadImage("colors.png");
  ok = loadImage("codeOk.png");
  ok2 = loadImage("codeOkHigh.png");
  type = loadImage("type.png");
  startGame = loadImage("start.png");
  startGame2 = loadImage("startHigh.png");
}
SetupMode::SetupMode() {
  for(int i=0;i<2;i++)
	games[i]=new Game();
  games[0]->type = TYPE_HUMAN;
  games[1]->type = TYPE_COMPUTER;

}
void SetupMode::display() {
  char str[256];
  int i;

  blit(background,0,0);

  if(state == -1) {
	/* Main menu */
	blit(title,400-title->w/2,150);
	
	blit(ply1,100,300+30*P1_TYPE-35);
	
	blit(type,150,300+30*P1_TYPE-5);
	sprintf(str,"%s",plyTypeNames[games[0]->type]);
	if(selection == P1_TYPE) drawText(str,500,300+30*P1_TYPE,0,255,255,255);
	else drawText(str,500,300+30*P1_TYPE,0,0,0,0);
	
	blit(code,150,300+30*P1_CODE-5);
	sprintf(str,"%s",isRandom[0]?"random":"customized");
	if(selection == P1_CODE) drawText(str,500,300+30*P1_CODE,0,255,255,255);
	else drawText(str,500,300+30*P1_CODE,0,0,0,0);
	
	blit(nColors,150,300+30*P1_COLORS-5);
	sprintf(str,"%d",games[0]->nColors);
	if(selection == P1_COLORS) drawText(str,500,300+30*P1_COLORS,0,255,255,255);
	else drawText(str,500,300+30*P1_COLORS,0,0,0,0);
	
	blit(ply2,100,300+30*P2_TYPE-35);
	blit(type,150,300+30*P2_TYPE-5);
	sprintf(str,"%s",plyTypeNames[games[1]->type]);
	if(selection == P2_TYPE) drawText(str,500,300+30*P2_TYPE,0,255,255,255);
	else drawText(str,500,300+30*P2_TYPE,0,0,0,0);
	
	blit(code,150,300+30*P2_CODE-5);
	sprintf(str,"%s",isRandom[1]?"random":"customized");
	if(selection == P2_CODE) drawText(str,500,300+30*P2_CODE,0,255,255,255);
	else drawText(str,500,300+30*P2_CODE,0,0,0,0);

	blit(nColors,150,300+30*P2_COLORS-5);
	sprintf(str,"%d",games[1]->nColors);
	if(selection == P2_COLORS) drawText(str,500,300+30*P2_COLORS,0,255,255,255);
	else drawText(str,500,300+30*P2_COLORS,0,0,0,0);
	
	if(selection == START) blit(startGame2,400-startGame2->w/2,300+30*START-5);
	else blit(startGame,400-startGame->w/2,300+30*START-5);

	/*	if(selection == START) drawText("Start",400,300+30*START,1,255,255,255);
		else drawText("Start",400,300+30*START,1,0,0,0);*/

	if(selection == QUIT) drawText("Quit",400,300+30*QUIT,1,255,255,255);
	else drawText("Quit",400,300+30*QUIT,1,0,0,0);

  } else {

	sprintf(str,"Select code for player %d",state+1);
	drawBigText(str,400,150,1,0,0,0);

	/* Select code for some player */
	if(selection == CODE_OK) blit(ok2,400-ok2->w/2,300+30*CODE_OK-5);
	else blit(ok,400-ok->w/2,300+30*CODE_OK-5);

	for(i=0;i<4;i++)
	blit(MainMode::colors[games[state]->code[i]],400-80+40*i,300);
  }
}
void SetupMode::idle(double dt) {
  int x,y;
  Uint8 mouseState=SDL_GetMouseState(&x,&y);
  selection=-1;
  if(y >= 300) selection = (y - 300) / 30;
}
void SetupMode::mouseDown(int button,int x,int y) {
  selection=-1;
  if(y >= 300) selection = (y - 300) / 30;
  if(state > -1 && selection == CODE_OK) { state=-1; return; }
  if(state > -1 && y > 300 && y < 348) {
	int xsel=(x-(400-80))/40;
	if(xsel >= 0 && xsel <= 3)
	  games[state]->code[xsel] = mymod(games[state]->code[xsel]+(button==1?1:-1),games[state]->nColors);
	return;
  }

  switch(selection) {
  case P1_TYPE: games[0]->type=mymod(games[0]->type+(button==1?1:-1),5); break;
  case P2_TYPE: games[1]->type=mymod(games[1]->type+(button==1?1:-1),5); break;
  case P1_COLORS: games[0]->nColors=mymod(games[0]->nColors-2+(button==1?1:-1),11)+2; break;
  case P2_COLORS: games[1]->nColors=mymod(games[1]->nColors-2+(button==1?1:-1),11)+2; break;
  case P1_CODE: if(button==1) { state=0; isRandom[0]=0; } else isRandom[0]=1; break;
  case P2_CODE: if(button==1) { state=1; isRandom[1]=0; } else isRandom[1]=1; break;
  case START: start(); break;
  case QUIT: exit(0); break;
  }
}
void SetupMode::activated() {
  int i;
  isRandom[0]=isRandom[1]=1;  
  for(i=0;i<2;i++)
	games[i]->reset();
  state=-1;
}
void SetupMode::start() {
  int i,j;

  for(i=0;i<2;i++)
	games[i]->isComputer=games[i]->type >= TYPE_AUTOMATA;

  GameMode::activate(MainMode::mainMode);
  
  for(i=0;i<2;i++) {
	MainMode::mainMode->games[i] = games[i];
	if(isRandom[i]) 
	  for(j=0;j<4;j++)
		games[i]->code[j] = rand() % games[i]->nColors;
  }
}
